@@ -1,13 +1,21 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
-__all__ = ("from_arrow",)
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
+
 import awkward as ak
-from awkward._layout import wrap_layout
-from awkward._nplikes.numpylike import NumpyMetadata
+from awkward._dispatch import high_level_function
+from awkward._layout import HighLevelContext
+from awkward._nplikes.numpy_like import NumpyMetadata
+
+__all__ = ("from_arrow",)
 
 np = NumpyMetadata.instance()
 
 
-def from_arrow(array, *, generate_bitmasks=False, highlevel=True, behavior=None):
+@high_level_function()
+def from_arrow(
+    array, *, generate_bitmasks=False, highlevel=True, behavior=None, attrs=None
+):
     """
     Args:
         array (`pyarrow.Array`, `pyarrow.ChunkedArray`, `pyarrow.RecordBatch`, or `pyarrow.Table`):
@@ -20,6 +28,8 @@ def from_arrow(array, *, generate_bitmasks=False, highlevel=True, behavior=None)
             a low-level #ak.contents.Content subclass.
         behavior (None or dict): Custom #ak.behavior for the output array, if
             high-level.
+        attrs (None or dict): Custom attributes for the output array, if
+            high-level.
 
     Converts an Apache Arrow array into an Awkward Array.
 
@@ -30,25 +40,22 @@ def from_arrow(array, *, generate_bitmasks=False, highlevel=True, behavior=None)
     low-level #ak.forms.Form), even through Parquet, making Parquet a good way to save
     Awkward Arrays for later use.
 
+    Because awkward uses numpy's dtype system, timestamp types do not have timezones.
+    If encountering timestamp types with timezones in the input arrow data, they
+    will be silently dropped.
+
     See also #ak.to_arrow, #ak.to_arrow_table, #ak.from_parquet, #ak.from_arrow_schema.
     """
-    with ak._errors.OperationErrorContext(
-        "ak.from_arrow",
-        {
-            "array": array,
-            "generate_bitmasks": generate_bitmasks,
-            "highlevel": highlevel,
-            "behavior": behavior,
-        },
-    ):
-        return _impl(array, generate_bitmasks, highlevel, behavior)
+    return _impl(array, generate_bitmasks, highlevel, behavior, attrs)
 
 
-def _impl(array, generate_bitmasks, highlevel, behavior):
+def _impl(array, generate_bitmasks, highlevel, behavior, attrs):
     import awkward._connect.pyarrow
 
     pyarrow = awkward._connect.pyarrow.pyarrow
 
+    ctx = HighLevelContext(behavior=behavior, attrs=attrs).finalize()
+
     out = awkward._connect.pyarrow.handle_arrow(
         array, generate_bitmasks=generate_bitmasks, pass_empty_field=True
     )
@@ -75,4 +82,4 @@ def _impl(array, generate_bitmasks, highlevel, behavior):
 
     ak._do.recursively_apply(out, remove_revertable)
 
-    return wrap_layout(out, behavior, highlevel)
+    return ctx.wrap(out, highlevel=highlevel)
