@@ -1,16 +1,31 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
-__all__ = ("var",)
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
+
 import awkward as ak
-from awkward._behavior import behavior_of
-from awkward._connect.numpy import unsupported
-from awkward._layout import maybe_posaxis
-from awkward._nplikes.numpylike import NumpyMetadata
+from awkward._attrs import attrs_of_obj
+from awkward._connect.numpy import UNSUPPORTED
+from awkward._dispatch import high_level_function
+from awkward._layout import (
+    HighLevelContext,
+    ensure_same_backend,
+    maybe_highlevel_to_lowlevel,
+    maybe_posaxis,
+)
+from awkward._namedaxis import (
+    NAMED_AXIS_KEY,
+    _get_named_axis,
+    _named_axis_to_positional_axis,
+)
+from awkward._nplikes.numpy_like import NumpyMetadata
 from awkward._regularize import regularize_axis
-from awkward._util import unset
+
+__all__ = ("nanvar", "var")
 
 np = NumpyMetadata.instance()
 
 
+@high_level_function()
 def var(
     x,
     weight=None,
@@ -19,7 +34,9 @@ def var(
     *,
     keepdims=False,
     mask_identity=False,
-    flatten_records=unset,
+    highlevel=True,
+    behavior=None,
+    attrs=None,
 ):
     """
     Args:
@@ -44,6 +61,12 @@ def var(
             empty lists results in None (an option type); otherwise, the
             calculation is followed through with the reducers' identities,
             usually resulting in floating-point `nan`.
+        highlevel (bool): If True, return an #ak.Array; otherwise, return
+            a low-level #ak.contents.Content subclass.
+        behavior (None or dict): Custom #ak.behavior for the output array, if
+            high-level.
+        attrs (None or dict): Custom attributes for the output array, if
+            high-level.
 
     Computes the variance in each group of elements from `x` (many
     types supported, including all Awkward Arrays and Records). The grouping
@@ -70,30 +93,16 @@ def var(
 
     See also #ak.nanvar.
     """
-    with ak._errors.OperationErrorContext(
-        "ak.var",
-        {
-            "x": x,
-            "weight": weight,
-            "ddof": ddof,
-            "axis": axis,
-            "keepdims": keepdims,
-            "mask_identity": mask_identity,
-        },
-    ):
-        if flatten_records is not unset:
-            message = (
-                "`flatten_records` is no longer a supported argument for reducers. "
-                "Instead, use `ak.ravel(array)` first to remove the record structure "
-                "and flatten the array."
-            )
-            if flatten_records:
-                raise ak._errors.wrap_error(ValueError(message))
-            else:
-                ak._errors.deprecate(message, "2.2.0")
-        return _impl(x, weight, ddof, axis, keepdims, mask_identity)
+    # Dispatch
+    yield x, weight
+
+    # Implementation
+    return _impl(
+        x, weight, ddof, axis, keepdims, mask_identity, highlevel, behavior, attrs
+    )
 
 
+@high_level_function()
 def nanvar(
     x,
     weight=None,
@@ -102,7 +111,9 @@ def nanvar(
     *,
     keepdims=False,
     mask_identity=True,
-    flatten_records=unset,
+    highlevel=True,
+    behavior=None,
+    attrs=None,
 ):
     """
     Args:
@@ -127,6 +138,12 @@ def nanvar(
             empty lists results in None (an option type); otherwise, the
             calculation is followed through with the reducers' identities,
             usually resulting in floating-point `nan`.
+        highlevel (bool): If True, return an #ak.Array; otherwise, return
+            a low-level #ak.contents.Content subclass.
+        behavior (None or dict): Custom #ak.behavior for the output array, if
+            high-level.
+        attrs (None or dict): Custom attributes for the output array, if
+            high-level.
 
     Like #ak.var, but treating NaN ("not a number") values as missing.
 
@@ -138,51 +155,49 @@ def nanvar(
 
     See also #ak.var.
     """
-    with ak._errors.OperationErrorContext(
-        "ak.nanvar",
-        {
-            "x": x,
-            "weight": weight,
-            "ddof": ddof,
-            "axis": axis,
-            "keepdims": keepdims,
-            "mask_identity": mask_identity,
-        },
-    ):
-        if flatten_records is not unset:
-            message = (
-                "`flatten_records` is no longer a supported argument for reducers. "
-                "Instead, use `ak.ravel(array)` first to remove the record structure "
-                "and flatten the array."
-            )
-            if flatten_records:
-                raise ak._errors.wrap_error(ValueError(message))
-            else:
-                ak._errors.deprecate(message, "2.2.0")
-        x = ak.operations.ak_nan_to_none._impl(x, False, None)
-        if weight is not None:
-            weight = ak.operations.ak_nan_to_none._impl(weight, False, None)
+    # Dispatch
+    yield x, weight
 
-        return _impl(x, weight, ddof, axis, keepdims, mask_identity)
-
-
-def _impl(x, weight, ddof, axis, keepdims, mask_identity):
-    axis = regularize_axis(axis)
-    behavior = behavior_of(x, weight)
-    x = ak.highlevel.Array(
-        ak.operations.to_layout(x, allow_record=False, allow_other=False),
-        behavior=behavior,
-    )
+    # Implementation
     if weight is not None:
-        weight = ak.highlevel.Array(
-            ak.operations.to_layout(weight, allow_record=False, allow_other=False),
-            behavior=behavior,
+        weight = ak.operations.ak_nan_to_none._impl(weight, True, behavior, attrs)
+
+    return _impl(
+        ak.operations.ak_nan_to_none._impl(x, highlevel, behavior, attrs),
+        weight,
+        ddof,
+        axis,
+        keepdims,
+        mask_identity,
+        highlevel,
+        behavior,
+        attrs,
+    )
+
+
+def _impl(x, weight, ddof, axis, keepdims, mask_identity, highlevel, behavior, attrs):
+    with HighLevelContext(behavior=behavior, attrs=attrs) as ctx:
+        x_layout, weight_layout = ensure_same_backend(
+            ctx.unwrap(x, allow_record=False, primitive_policy="error"),
+            ctx.unwrap(
+                weight,
+                allow_record=False,
+                allow_unknown=False,
+                primitive_policy="error",
+                none_policy="pass-through",
+            ),
         )
 
+    x = ctx.wrap(x_layout)
+    weight = ctx.wrap(weight_layout, allow_other=True)
+
+    # Handle named axis
+    named_axis = _get_named_axis(ctx)
+    # Step 1: Normalize named axis to positional axis
+    axis = _named_axis_to_positional_axis(named_axis, axis)
+    axis = regularize_axis(axis, none_allowed=True)
+
     with np.errstate(invalid="ignore", divide="ignore"):
-        xmean = ak.operations.ak_mean._impl(
-            x, weight, axis, keepdims=True, mask_identity=True
-        )
         if weight is None:
             sumw = ak.operations.ak_count._impl(
                 x,
@@ -190,15 +205,26 @@ def _impl(x, weight, ddof, axis, keepdims, mask_identity):
                 keepdims=True,
                 mask_identity=True,
                 highlevel=True,
-                behavior=None,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
             )
-            sumwxx = ak.operations.ak_sum._impl(
-                (x - xmean) ** 2,
+            sumwx = ak.operations.ak_sum._impl(
+                x,
                 axis,
                 keepdims=True,
                 mask_identity=True,
                 highlevel=True,
-                behavior=None,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
+            )
+            sumwxx = ak.operations.ak_sum._impl(
+                x * x,
+                axis,
+                keepdims=True,
+                mask_identity=True,
+                highlevel=True,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
             )
         else:
             sumw = ak.operations.ak_sum._impl(
@@ -207,23 +233,41 @@ def _impl(x, weight, ddof, axis, keepdims, mask_identity):
                 keepdims=True,
                 mask_identity=True,
                 highlevel=True,
-                behavior=None,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
             )
-            sumwxx = ak.operations.ak_sum._impl(
-                (x - xmean) ** 2 * weight,
+            sumwx = ak.operations.ak_sum._impl(
+                x * weight,
                 axis,
                 keepdims=True,
                 mask_identity=True,
                 highlevel=True,
-                behavior=None,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
             )
+            sumwxx = ak.operations.ak_sum._impl(
+                x * x * weight,
+                axis,
+                keepdims=True,
+                mask_identity=True,
+                highlevel=True,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
+            )
+        mean = sumwx / sumw
+        out = sumwxx / sumw - mean * mean
         if ddof != 0:
-            out = (sumwxx / sumw) * (sumw / (sumw - ddof))
-        else:
-            out = sumwxx / sumw
+            out = out * (sumw / (sumw - ddof))
 
         if not mask_identity:
-            out = ak.highlevel.Array(ak.operations.fill_none(out, np.nan, axis=-1))
+            out = ak.operations.fill_none(
+                out,
+                np.nan,
+                axis=-1,
+                behavior=ctx.behavior,
+                attrs=ctx.attrs,
+                highlevel=True,
+            )
 
         if axis is None:
             if not keepdims:
@@ -233,19 +277,32 @@ def _impl(x, weight, ddof, axis, keepdims, mask_identity):
                 posaxis = maybe_posaxis(out.layout, axis, 1)
                 out = out[(slice(None, None),) * posaxis + (0,)]
 
-        return out
+        wrapped = ctx.without_attr(NAMED_AXIS_KEY).wrap(
+            maybe_highlevel_to_lowlevel(out),
+            highlevel=highlevel,
+            allow_other=True,
+        )
+
+        # propagate named axis to output
+        return ak.operations.ak_with_named_axis._impl(
+            wrapped,
+            named_axis=_get_named_axis(attrs_of_obj(out), allow_any=True),
+            highlevel=highlevel,
+            behavior=None,
+            attrs=None,
+        )
 
 
 @ak._connect.numpy.implements("var")
 def _nep_18_impl_var(
     a,
     axis=None,
-    dtype=unsupported,
-    out=unsupported,
+    dtype=UNSUPPORTED,
+    out=UNSUPPORTED,
     ddof=0,
     keepdims=False,
     *,
-    where=unsupported,
+    where=UNSUPPORTED,
 ):
     return var(a, axis=axis, keepdims=keepdims, ddof=ddof)
 
@@ -254,11 +311,11 @@ def _nep_18_impl_var(
 def _nep_18_impl_nanvar(
     a,
     axis=None,
-    dtype=unsupported,
-    out=unsupported,
+    dtype=UNSUPPORTED,
+    out=UNSUPPORTED,
     ddof=0,
     keepdims=False,
     *,
-    where=unsupported,
+    where=UNSUPPORTED,
 ):
     return nanvar(a, axis=axis, keepdims=keepdims, ddof=ddof)
