@@ -1,4 +1,6 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
 
 import itertools
 
@@ -36,9 +38,9 @@ def test_numpyarray():
             .to_typetracer()
             ._mergemany(
                 [
-                    ak.contents.NumpyArray(two),
-                    ak.contents.NumpyArray(three),
-                    ak.contents.NumpyArray(four),
+                    ak.contents.NumpyArray(two).to_typetracer(),
+                    ak.contents.NumpyArray(three).to_typetracer(),
+                    ak.contents.NumpyArray(four).to_typetracer(),
                 ]
             )
             .form
@@ -69,9 +71,9 @@ def test_numpyarray():
             .to_typetracer()
             ._mergemany(
                 [
-                    ak.contents.NumpyArray(two),
-                    ak.contents.EmptyArray(),
-                    ak.contents.NumpyArray(four),
+                    ak.contents.NumpyArray(two).to_typetracer(),
+                    ak.contents.EmptyArray().to_typetracer(),
+                    ak.contents.NumpyArray(four).to_typetracer(),
                 ]
             )
             .form
@@ -113,11 +115,15 @@ def test_lists():
         [4.0, 5.0],
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
     assert (
-        four.to_typetracer()._mergemany([three, four, one]).form
+        four.to_typetracer()
+        ._mergemany([three.to_typetracer(), four.to_typetracer(), one.to_typetracer()])
+        .form
         == four._mergemany([three, four, one]).form
     )
 
@@ -145,12 +151,16 @@ def test_lists():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
     assert (
-        four.to_typetracer()._mergemany([three, four, one]).form
+        four.to_typetracer()
+        ._mergemany([three.to_typetracer(), four.to_typetracer(), one.to_typetracer()])
+        .form
         == four._mergemany([three, four, one]).form
     )
 
@@ -173,7 +183,9 @@ def test_records():
         {"x": 7, "y": [1, 2]},
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
@@ -188,7 +200,9 @@ def test_records():
         {"x": 7, "y": [1, 2]},
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
@@ -208,7 +222,9 @@ def test_tuples():
         (7, [1, 2]),
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
@@ -223,7 +239,9 @@ def test_tuples():
         (7, [1, 2]),
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
@@ -250,7 +268,9 @@ def test_indexed():
         None,
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
@@ -261,7 +281,9 @@ def test_reverse_indexed():
     three = ak.highlevel.Array([None, 6, None]).layout
     assert to_list(one._mergemany([two, three])) == [1, 2, 3, 4, 5, None, 6, None]
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
 
@@ -282,7 +304,9 @@ def test_reverse_indexed():
         9,
     ]
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
 
@@ -413,27 +437,39 @@ def test_bytemasked():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
     assert (
-        four.to_typetracer()._mergemany([three, two, one]).form
+        four.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer(), one.to_typetracer()])
+        .form
         == four._mergemany([three, two, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([four, one]).form
+        three.to_typetracer()
+        ._mergemany([four.to_typetracer(), one.to_typetracer()])
+        .form
         == three._mergemany([four, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([four, one, two]).form
+        three.to_typetracer()
+        ._mergemany([four.to_typetracer(), one.to_typetracer(), two.to_typetracer()])
+        .form
         == three._mergemany([four, one, two]).form
     )
     assert (
-        three.to_typetracer()._mergemany([two, one]).form
+        three.to_typetracer()
+        ._mergemany([two.to_typetracer(), one.to_typetracer()])
+        .form
         == three._mergemany([two, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([two, one, four]).form
+        three.to_typetracer()
+        ._mergemany([two.to_typetracer(), one.to_typetracer(), four.to_typetracer()])
+        .form
         == three._mergemany([two, one, four]).form
     )
 
@@ -458,15 +494,29 @@ def test_empty():
         == one._mergemany([two]).form
     )
     assert (
-        one.to_typetracer()._mergemany([two, one, two, one, two]).form
+        one.to_typetracer()
+        ._mergemany(
+            [
+                two.to_typetracer(),
+                one.to_typetracer(),
+                two.to_typetracer(),
+                one.to_typetracer(),
+                two.to_typetracer(),
+            ]
+        )
+        .form
         == one._mergemany([two, one, two, one, two]).form
     )
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
     assert (
-        one.to_typetracer()._mergemany([two, three, four]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([two, three, four]).form
     )
     assert (
@@ -474,19 +524,27 @@ def test_empty():
         == one._mergemany([three]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, four]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([three, four]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, two]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer()])
+        .form
         == one._mergemany([three, two]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, two, four]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer(), four.to_typetracer()])
+        .form
         == one._mergemany([three, two, four]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, four, two]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), four.to_typetracer(), two.to_typetracer()])
+        .form
         == one._mergemany([three, four, two]).form
     )
 
@@ -582,27 +640,39 @@ def test_union():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, two]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer()])
+        .form
         == one._mergemany([three, two]).form
     )
     assert (
-        two.to_typetracer()._mergemany([one, three]).form
+        two.to_typetracer()
+        ._mergemany([one.to_typetracer(), three.to_typetracer()])
+        .form
         == two._mergemany([one, three]).form
     )
     assert (
-        two.to_typetracer()._mergemany([three, one]).form
+        two.to_typetracer()
+        ._mergemany([three.to_typetracer(), one.to_typetracer()])
+        .form
         == two._mergemany([three, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([one, two]).form
+        three.to_typetracer()
+        ._mergemany([one.to_typetracer(), two.to_typetracer()])
+        .form
         == three._mergemany([one, two]).form
     )
     assert (
-        three.to_typetracer()._mergemany([two, one]).form
+        three.to_typetracer()
+        ._mergemany([two.to_typetracer(), one.to_typetracer()])
+        .form
         == three._mergemany([two, one]).form
     )
 
@@ -698,27 +768,39 @@ def test_union_option():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, two]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer()])
+        .form
         == one._mergemany([three, two]).form
     )
     assert (
-        two.to_typetracer()._mergemany([one, three]).form
+        two.to_typetracer()
+        ._mergemany([one.to_typetracer(), three.to_typetracer()])
+        .form
         == two._mergemany([one, three]).form
     )
     assert (
-        two.to_typetracer()._mergemany([three, one]).form
+        two.to_typetracer()
+        ._mergemany([three.to_typetracer(), one.to_typetracer()])
+        .form
         == two._mergemany([three, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([one, two]).form
+        three.to_typetracer()
+        ._mergemany([one.to_typetracer(), two.to_typetracer()])
+        .form
         == three._mergemany([one, two]).form
     )
     assert (
-        three.to_typetracer()._mergemany([two, one]).form
+        three.to_typetracer()
+        ._mergemany([two.to_typetracer(), one.to_typetracer()])
+        .form
         == three._mergemany([two, one]).form
     )
 
@@ -812,27 +894,39 @@ def test_union_option():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, two]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer()])
+        .form
         == one._mergemany([three, two]).form
     )
     assert (
-        two.to_typetracer()._mergemany([one, three]).form
+        two.to_typetracer()
+        ._mergemany([one.to_typetracer(), three.to_typetracer()])
+        .form
         == two._mergemany([one, three]).form
     )
     assert (
-        two.to_typetracer()._mergemany([three, one]).form
+        two.to_typetracer()
+        ._mergemany([three.to_typetracer(), one.to_typetracer()])
+        .form
         == two._mergemany([three, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([one, two]).form
+        three.to_typetracer()
+        ._mergemany([one.to_typetracer(), two.to_typetracer()])
+        .form
         == three._mergemany([one, two]).form
     )
     assert (
-        three.to_typetracer()._mergemany([two, one]).form
+        three.to_typetracer()
+        ._mergemany([two.to_typetracer(), one.to_typetracer()])
+        .form
         == three._mergemany([two, one]).form
     )
 
@@ -926,27 +1020,39 @@ def test_union_option():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
     assert (
-        one.to_typetracer()._mergemany([three, two]).form
+        one.to_typetracer()
+        ._mergemany([three.to_typetracer(), two.to_typetracer()])
+        .form
         == one._mergemany([three, two]).form
     )
     assert (
-        two.to_typetracer()._mergemany([one, three]).form
+        two.to_typetracer()
+        ._mergemany([one.to_typetracer(), three.to_typetracer()])
+        .form
         == two._mergemany([one, three]).form
     )
     assert (
-        two.to_typetracer()._mergemany([three, one]).form
+        two.to_typetracer()
+        ._mergemany([three.to_typetracer(), one.to_typetracer()])
+        .form
         == two._mergemany([three, one]).form
     )
     assert (
-        three.to_typetracer()._mergemany([one, two]).form
+        three.to_typetracer()
+        ._mergemany([one.to_typetracer(), two.to_typetracer()])
+        .form
         == three._mergemany([one, two]).form
     )
     assert (
-        three.to_typetracer()._mergemany([two, one]).form
+        three.to_typetracer()
+        ._mergemany([two.to_typetracer(), one.to_typetracer()])
+        .form
         == three._mergemany([two, one]).form
     )
 
@@ -969,7 +1075,9 @@ def test_strings():
     ]
 
     assert (
-        one.to_typetracer()._mergemany([two, three]).form
+        one.to_typetracer()
+        ._mergemany([two.to_typetracer(), three.to_typetracer()])
+        .form
         == one._mergemany([two, three]).form
     )
 
